/*
 * memory.c
 *
 * Copyright (c) 2014-2015 Kirill Kryukov
 *
 * This file is a part of the Nucleotide Sequence Clusterizer source code,
 * and may only be used, modified, and distributed under the terms of the
 * license, located in the file LICENSE.txt within this package.
 */

#ifdef INCLUDED_FROM_ALL_C

void* malloc1(const size_t size)
{
    void *buf;
#undef malloc
    buf = malloc(size);
#define malloc  DONT_USE_MALLOC
    if (!buf)
    {
        fprintf(stderr,"Error: can't allocate %" PRINT_SIZE_T " bytes\n",size);
        exit(10);
    }
    allocated += size;
    if (allocated > max_allocated) { max_allocated = allocated; }
    return buf;
}

void* calloc1(const size_t nelem,const size_t elsize)
{
    void *buf;
#undef calloc
    buf = calloc(nelem,elsize);
#define calloc  DONT_USE_CALLOC
    if (!buf)
    {
        fprintf(stderr,"Error: can't allocate %" PRINT_SIZE_T " bytes\n",nelem*elsize);
        exit(11);
    }
    allocated += nelem*elsize;
    if (allocated > max_allocated) { max_allocated = allocated; }
    return buf;
}

void* realloc1(void *data,const size_t new_size,const size_t old_size)
{
    void *buf;
#undef realloc
    buf = realloc(data,new_size);
#define realloc DONT_USE_REALLOC
    if (!buf)
    {
        fprintf(stderr,"Error: can't reallocate %" PRINT_SIZE_T " bytes into %" PRINT_SIZE_T " bytes\n",old_size,new_size);
        exit(12);
    }
    allocated -= old_size;
    allocated += new_size;
    if (allocated > max_allocated) { max_allocated = allocated; }
    return buf;
}

void free1(void *data,const size_t size)
{
#undef free
    free(data);
#define free    DONT_USE_FREE
    allocated -= size;
}

#endif
